/* Copyright (C) 1999-2000 Hans Petter K. Jansson
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * You can contact the library's author by sending e-mail to <hpj@styx.net>.
 */

#ifndef _NVTP_H
#define _NVTP_H 1

#ifndef LIBFLUX_BUILD
#  include <flux/sock.h>
#  include <flux/tt.h>
#else
#  include "sock.h"
#  include "tt.h"
#endif /* LIBFLUX_BUILD */


enum nvtp_state
{
  NVTP_STATE_NONE = 0, NVTP_STATE_LINE_DATA, NVTP_STATE_MAYBE_CR,
  NVTP_STATE_MAYBE_LF
};


#define NVTP_STATE_CR NVTP_STATE_MAYBE_CR
#define NVTP_STATE_LF NVTP_STATE_MAYBE_LF


typedef struct _nvtp
{
  SOCK *s;
  
  struct _nvtp *parent;
  
  TT *queue_in, *queue_out;

  TT *root_in, *root_out;
  TT *node_in, *node_out;

  u16 byte_out;
  
  u16 state_in : 2;
  u16 state_out : 2;
}
NVTP;


/* --- */

NVTP *_nvtp_alloc();
NVTP *nvtp_new();
NVTP *nvtp_pipe_new();
NVTP *nvtp_new_with_port(int port);
NVTP *nvtp_accepting_new(int port);
NVTP *nvtp_new_with_sock(SOCK *s);
void nvtp_del(NVTP *nvtp);

#define nvtp_get_parent(nvtp) (nvtp)->parent

#define nvtp_queue_out_len(nvtp) tt_count_children(((NVTP *) nvtp)->queue_out)
#define nvtp_queue_in_len(nvtp) tt_count_children(((NVTP *) nvtp)->queue_in)

#define nvtp_buf_out_size(nvtp) sock_buf_out_size(((NVTP *) nvtp)->s)
#define nvtp_buf_in_size(nvtp) sock_buf_in_size(((NVTP *) nvtp)->s)

#define nvtp_get_sock(nvtp) (((NVTP *) nvtp)->s)
#define nvtp_get_fd_read(nvtp) sock_get_fd_read(nvtp_get_sock((NVTP *) nvtp))
#define nvtp_get_fd_write(nvtp) sock_get_fd_write(nvtp_get_sock((NVTP *) nvtp))

#define nvtp_is_connecting(nvtp) sock_is_connecting(((NVTP *) nvtp)->s)
#define nvtp_is_connected(nvtp) sock_is_connected(((NVTP *) nvtp)->s)
#define nvtp_is_accepting(nvtp) sock_is_accepting(((NVTP *) nvtp)->s)
#define nvtp_is_active(nvtp) (nvtp_is_connected(nvtp) | nvtp_is_accepting(nvtp))

NVTP *nvtp_accept(NVTP *nvtp);

void nvtp_push(NVTP *nvtp);
void nvtp_pull(NVTP *nvtp);

void nvtp_enqueue(NVTP *nvtp, char *str);
char *nvtp_dequeue(NVTP *nvtp);

void nvtp_enqueue_tt(NVTP *nvtp, TT *tt);
TT *nvtp_dequeue_tt(NVTP *nvtp);

void nvtp_send(NVTP *nvtp, char *str);
char *nvtp_recv(NVTP *nvtp);
void nvtp_flush(NVTP *nvtp);


#endif /* _NVTP_H */
